"""Dummy data model definition."""

from sqlalchemy import Column, Integer, String, Date, Float, UniqueConstraint
from sqlalchemy.orm import declarative_base

Base = declarative_base()


class StockValue(Base):
    """Stock value data model."""

    __tablename__ = "stock_value"
    id = Column(Integer, primary_key=True)
    symbol = Column(String)
    date = Column(Date)
    open = Column(Float)
    high = Column(Float)
    low = Column(Float)
    close = Column(Float)
    __table_args__ = (UniqueConstraint('symbol', 'date'),)

    def __repr__(self):
        return f"<StockValue(symbol='{self.symbol}', ...)>"



"""Script to create DB tables."""
from models import Base
from sqlalchemy import create_engine

import os

PG_USER = os.getenv('PG_USER')
PG_PASSWORD = os.getenv('PG_PASSWORD')
PG_HOST = os.getenv('PG_HOST')
PG_PORT = "5432"
PG_DB = os.getenv('PG_DB')
URI = f"postgresql://{PG_USER}:{PG_PASSWORD}@{PG_HOST}:{PG_PORT}/{PG_DB}"


def main():
    """Program entrypoint."""
    engine = create_engine(URI, echo=True)
    Base.metadata.create_all(engine)


if __name__ == "__main__":
    main()



from db_api_client.parent_cli import DBAPIClient


class PostgresClient(DBAPIClient):
    def __init__(self, user, password, host, port, database):
        db = f'//{user}:{password}@{host}:{port}/{database}'
        super().__init__('postgresql', db)


if __name__ == '__main__':
    pg_user = 'postgres'
    pg_password = 'postgres'
    pg_host = 'localhost'
    pg_port = 5432
    pg_database = 'stock'
    postgres_cli = PostgresClient(pg_user, pg_password, pg_host, pg_port, pg_database)
    print(postgres_cli.to_frame('SELECT * FROM stock_value'))



import pandas as pd
from sqlalchemy import create_engine


class DBAPIClient:
    def __init__(self, dialect, db):
        self.dialect = dialect
        self.db = db
        self._engine = None

    def _get_engine(self):
        db_uri = f'{self.dialect}:{self.db}'
        print(db_uri)
        if not self._engine:
            self._engine = create_engine(db_uri)
        return self._engine

    def _connect(self):
        return self._get_engine().connect()

    @staticmethod
    def _cursor_columns(cursor):
        if hasattr(cursor, 'keys'):
            return cursor.keys()
        else:
            return [c[0] for c in cursor.description]

    def execute(self, sql, connection=None):
        if connection is None:
            connection = self._connect()
        return connection.execute(sql)

    def insert_from_frame(self, df, table, if_exists='append', index=False, **kwargs):
        connection = self._connect()
        with connection:
            df.to_sql(table, connection, if_exists=if_exists, index=index, **kwargs)

    def to_frame(self, *args, **kwargs):
        cursor = self.execute(*args, **kwargs)
        if not cursor:
            return
        data = cursor.fetchall()
        if data:
            df = pd.DataFrame(data, columns=self._cursor_columns(cursor))
        else:
            df = pd.DataFrame()
        return df



from db_api_client.parent_cli import DBAPIClient


class SqLiteClient(DBAPIClient):
    def __init__(self, db):
        super().__init__('sqlite', db)


if __name__ == '__main__':
    sqlite_db = '////tmp/sqlite_default.db'
    sqlite_cli = SqLiteClient(sqlite_db)
    print(sqlite_cli.to_frame('SELECT * FROM stocks_daily'))
